Class: ArchSpec::Edge
- Inherits:
-
Data
- Object
- Data
- ArchSpec::Edge
- Defined in:
- lib/archspec/model.rb
Constant Summary collapse
- VERBS =
{ references_constant: 'references', inherits_from: 'inherits from', includes: 'includes', prepends: 'prepends', extends: 'extends', calls_named_method: 'calls', instantiates_and_invokes: 'instantiates and invokes', requires: 'requires', requires_relative: 'requires', dynamic_feature: 'uses dynamic feature' }.freeze
Instance Attribute Summary collapse
-
#confidence ⇒ Object
readonly
Returns the value of attribute confidence.
-
#from_constant ⇒ Object
readonly
Returns the value of attribute from_constant.
-
#from_path ⇒ Object
readonly
Returns the value of attribute from_path.
-
#lexical_nesting ⇒ Object
readonly
Returns the value of attribute lexical_nesting.
-
#location ⇒ Object
readonly
Returns the value of attribute location.
-
#receiver ⇒ Object
readonly
Returns the value of attribute receiver.
-
#receiver_scope ⇒ Object
readonly
Returns the value of attribute receiver_scope.
-
#resolved_method ⇒ Object
readonly
Returns the value of attribute resolved_method.
-
#resolved_receiver ⇒ Object
readonly
Returns the value of attribute resolved_receiver.
-
#resolved_to ⇒ Object
readonly
Returns the value of attribute resolved_to.
-
#to ⇒ Object
readonly
Returns the value of attribute to.
-
#type ⇒ Object
readonly
Returns the value of attribute type.
Instance Method Summary collapse
-
#verb ⇒ Object
The edge type as prose, for diagnostics and explain output.
Instance Attribute Details
#confidence ⇒ Object (readonly)
Returns the value of attribute confidence
129 130 131 |
# File 'lib/archspec/model.rb', line 129 def confidence @confidence end |
#from_constant ⇒ Object (readonly)
Returns the value of attribute from_constant
129 130 131 |
# File 'lib/archspec/model.rb', line 129 def from_constant @from_constant end |
#from_path ⇒ Object (readonly)
Returns the value of attribute from_path
129 130 131 |
# File 'lib/archspec/model.rb', line 129 def from_path @from_path end |
#lexical_nesting ⇒ Object (readonly)
Returns the value of attribute lexical_nesting
129 130 131 |
# File 'lib/archspec/model.rb', line 129 def lexical_nesting @lexical_nesting end |
#location ⇒ Object (readonly)
Returns the value of attribute location
129 130 131 |
# File 'lib/archspec/model.rb', line 129 def location @location end |
#receiver ⇒ Object (readonly)
Returns the value of attribute receiver
129 130 131 |
# File 'lib/archspec/model.rb', line 129 def receiver @receiver end |
#receiver_scope ⇒ Object (readonly)
Returns the value of attribute receiver_scope
129 130 131 |
# File 'lib/archspec/model.rb', line 129 def receiver_scope @receiver_scope end |
#resolved_method ⇒ Object (readonly)
Returns the value of attribute resolved_method
129 130 131 |
# File 'lib/archspec/model.rb', line 129 def resolved_method @resolved_method end |
#resolved_receiver ⇒ Object (readonly)
Returns the value of attribute resolved_receiver
129 130 131 |
# File 'lib/archspec/model.rb', line 129 def resolved_receiver @resolved_receiver end |
#resolved_to ⇒ Object (readonly)
Returns the value of attribute resolved_to
129 130 131 |
# File 'lib/archspec/model.rb', line 129 def resolved_to @resolved_to end |
#to ⇒ Object (readonly)
Returns the value of attribute to
129 130 131 |
# File 'lib/archspec/model.rb', line 129 def to @to end |
#type ⇒ Object (readonly)
Returns the value of attribute type
129 130 131 |
# File 'lib/archspec/model.rb', line 129 def type @type end |
Instance Method Details
#verb ⇒ Object
The edge type as prose, for diagnostics and explain output.
157 158 159 |
# File 'lib/archspec/model.rb', line 157 def verb VERBS.fetch(type, type.to_s.tr('_', ' ')) end |